# Moritz Orbach 2005, 2006, 2007, 2008, 2010, 2011, 2021
# gnu@apfelboymchen.net
# http://apfelboymchen.net/gnu/configstuff/scripts/
# Moritz Orbach, 2021
#
# Some functions useful in daily shell life

# history grep
hgrep()
{
  history | grep -ai -- "$@" | tail -n42
}

# find big files in the current file system (default: 50 MB)
findbig()
{
  local meg="${1-50}"
  find -xdev -size +$(($meg*1024))k -exec du -h {} \; 2>/dev/null
}

# generate a cryptic password
ddpw()
{
  local count="${1-20}"
  if ! echo "$count" | grep -q '^[0-9]\+$'; then
    echo "invalid count" >&2
    echo "usage: ddpw [COUNT]" >&2
    return 0
  fi
  local i
  for ((i = 0; i < $count; i++)); do
    strings < /dev/urandom | tr -d "'\`[:space:]\\\:/&|" | dd bs=1c count="$count" 2>/dev/null
    echo
  done
}

# WARNING: REMOTE HOST IDENTIFICATION HAS CHANGED!
delhost()
{
  local f=~/.ssh/known_hosts
  if [[ "$1" =~ ^[0-9]+$ ]]; then
    test -f "$f" && sed -i "$1d" "$f"
  else
    echo "invalid line number" >&2
    echo "usage: delhost LINE" >&2
    echo "deletes line LINE from ~/.ssh/known_hosts" >&2
  fi
}

function x509()
{
  if [ -e "$1" ]; then
    local f
    for f in "$@"; do
      openssl x509 -text -noout -in "$f" -certopt "no_extensions,no_sigdump,no_pubkey"
    done
  else
    echo "usage: x509 FILE" >&2
    echo "prints some relevant info of a x509 certificate (openssl x509)" >&2
  fi
}

# top für arme
mtop()
{
  ps -e -o user,pid,stime,ppid,pcpu,nice,vsz,rss,pmem,args | sort -b -k9 | less
}

# make path short (i think newer bash has something similiar in PS1)
sp()
{   
    PS1=$(echo "$PS1" | sed 's:\\w:<\\W:')
}

# make path long again
lp() 
{ 
    PS1=$(echo "$PS1" | sed 's:<\\W:\\w:')
}

# short ls
sls()
{
  \ls -x --group-directories-first --color "$@"
}

# files lowern
low()
{
   for file in "$@"
   do
     lowfile=`echo "$file" | tr '[:upper:]' '[:lower:]'`
     mv "$file" "$lowfile"
   done
}

mclock()
{ 
  test "$1" && local add="-d"
  /bin/date +'%a, %d.%m.%Y (%b) %T %z %Z' $add $1
}

calc () 
{ 
    echo "scale=4;$@" | bc -l
}

zipdir()
{
  local z
  for z in *.zip; do unzip -d "${z%%.zip}" "$z"; done
}

rardir()
{
  local i
  local x
  for i in *.rar;do x="${i%%.rar}";mkdir "$x";cd "$x";unrar x ../"$i" || read -sn1 -p "warnix" ;cd ..;done
}

# the environ file contains the environment as it looked when the process was 
# spawned. That means that any changes the process might have made to its 
# environment will not be visible
# -> wtf wo steht das? Nicht in /usr/src/linux/Documentation/filesystems/proc.rst
# Aber ist nachvollziehbar!
# envvars werden auf dem stack übergeben und von glibc in den heap kopiert.
# Das muss die stack-Ansicht sein, von heap-inhalten weiß der kernel ja nix.
# https://unix.stackexchange.com/questions/29128/how-to-read-environment-variables-of-a-process
envof()
{
  local pid="$1"
  local proc="/proc/$pid/environ"
  test -e "$proc" || { echo "pidenv: $pid: not found" >&2; return; }
  tr '\0' '\n' < "$proc"
}

stracepid()
{
  strace -p $(pgrep "\<$@\>") 2>&1
}

ltracepid()
{
  ltrace -p $(pgrep "\<$@\>") 2>&1
}

# edit iptables rules in your favourte $VISUAL
vifw()
{
    local editor="${VISUAL-vi}"
    tmpfile="$(mktemp /tmp/vifw_XXXXXXXXXXX)"  || exit 1
    while :; do # just a "clever" way to unlink $tmpfile in every case
        iptables-save -c >> "$tmpfile"         || break
        old_md5="$(md5sum "$tmpfile")"         || break
        old_lastmod="$(stat -c %Y "$tmpfile")" || break
        $editor "$tmpfile"                     || break
        new_lastmod="$(stat -c %Y "$tmpfile")" || break
        new_md5="$(md5sum "$tmpfile")"         || break
        if [ $new_lastmod -gt $old_lastmod -a "$old_md5" != "$new_md5" ]; then
            echo iptables-restore -c "$tmpfile"
        else
            echo "ruleset unchanged"
        fi
        break
    done
    rm -f "$tmpfile"
}

# vim:sw=2:ts=2
# birth: old
